//Hardware is a neopixel 24-led ring and an Uno
//Note - When I tried this on a Nano iot 33 the animation flickered.
//Simple tests showed addressing up-chain LEDs would "land" at the wrong LED., e.g. writing to LED 23 would sometimes affect LED 10
//Forums report chips with interrupts from BLE and wifi may affect timing.
//Stock adafruit library might work better in that regard? But I like FastLED color handling. So... Uno for now

#include <Ramp.h>
#include "FastLED.h"
#define NUM_LEDS 24
#define DATA_PIN 3

//
enum { CHASE, DISCO, BLUE} state = BLUE;

#define TASKER_MAX_TASKS 4
#include "Tasker.h"

//Although the LEDs are physically RGB, most of the color references will be HSV.
CRGB leds[NUM_LEDS];

//The Tasker system will be used to update the background color periodically
Tasker tasker;
boolean update = true;

//The Ramp library will animate several things:
// - the position of two points, specified as a single float dimension around a 24-led ring
// - the hue of one of the points
// - the hue of the background

//One point, with a changing hue
rampFloat pos = 0.0;
rampByte hue;

//A second point, with a fixed hue
rampFloat pos2 = 23;

//A background hue
rampByte bg = 20;

void setup() {

  Serial.begin(9600);

  //FastLED init
  FastLED.addLeds<NEOPIXEL, DATA_PIN>(leds, NUM_LEDS);

  //the newBG function will get called four seconds from now
  tasker.setTimeout(newBG, 4000);
  tasker.setInterval(nextState, 8000);

  //These values will ease between their initial position and the target
  pos.go(48.0, 10000, QUARTIC_INOUT, BACKANDFORTH);
  pos2.go(0, 8000, CUBIC_INOUT, BACKANDFORTH);
  hue.go(255.0, 8000, SINUSOIDAL_INOUT, BACKANDFORTH);
}


//This is the full loop with ramp and tasker
void loop() {
  FastLED.clear(); //clear the LEDs

  switch (state) {
    case CHASE: //two leds running around on a changing background
      for (int i = 0; i < NUM_LEDS; i++) {
        //set background color:
        //Change the color of the 'background' e.g. all LEDs
        leds[i].setHSV(bg.update(), 200, 64);
      }

      //Draw colored points at different locations
      setPos(pos.update(), hue.update());
      setPos(pos2.update(), 100);
      FastLED.show();
      break;

    case DISCO: //strobe flashes
      for (int i = 0; i < NUM_LEDS; i++) {
        //set background color:
        //Change the color of the 'background' e.g. all LEDs
        leds[i].setHSV(200, 200, 20);
      }
      FastLED.show();
      delay(100);
      for (int i = 0; i < NUM_LEDS; i++) {
        //set background color:
        //Change the color of the 'background' e.g. all LEDs
        leds[i].setHSV(200, 200, 255);
      }
      FastLED.show();
      delay(100);
      break;

    case BLUE: //sparkling shades of blue
      for (int i = 0; i < NUM_LEDS; i++) {
        //set background color:
        //Change the color of the 'background' e.g. all LEDs
        leds[i].setHSV(random(140, 180), 255, random(50, 100));
      }
      FastLED.show();
      break;
  }

  tasker.loop(); //this needs to be called frequently to keep tasker going.
}

//Change the background color
void newBG() {
  bg.go(random(255), 1000, LINEAR); //Pick a new hue, and start going to it with a 1-second linear transition
  tasker.setTimeout(newBG, random(2000, 6000)); //re-call newBG a rando time the future. If wanted the same timing we could use setInterval
}

//Set pos takes a position and hue argument.
//It 'draws' a point at position of color hue.
//The trick is it can do "fractional" positions, cross fading between
//adjacent LEDs to give a smooth movement best suited for demonstrating Ramp.
//Fractional positions work like:
//e.g. position = 1.75:
//   - led 1 @ 25% and led 2 @75%
//This probably isn't strictly speaking correct - there's probably some neat luminance model for croass fading.
// But this looks OK
void setPos(float p, int hue) {
  int basePosition = int(p) % NUM_LEDS; //Wrap around 23->0
  int nextPosition = (basePosition + 1) % NUM_LEDS; //same
  float fraction = p - (int)p; //the fractional part

  //These serial statements help visualize how the setPos function works
  //But only use when updating one position
  //  Serial.print(p);
  //  Serial.print(',');
  //  Serial.print(basePosition);
  //  Serial.print(',');
  //  Serial.print(nextPosition);
  //  Serial.print(',');
  //  Serial.println(fraction);

  //TODO: read current bg color and blend in point color.
  //As is, not doing this makes the edges of a traveling point momentarilly too dark.
  //Alpha would be nice but I didn't see it in FastLED's docs.
  leds[basePosition].setHSV( hue, 255, (1 - fraction) * 255);
  leds[nextPosition].setHSV( hue, 255, fraction * 255);
}

//set the global state to the next state. Do a short fade before continuing - nice transition!
void nextState() {
  if (state == CHASE) state = DISCO;
  else if (state == DISCO) state = BLUE;
  else state = CHASE;

  //fase out for next state
  for (int i = 0; i < 255; i++) {
    fadeToBlackBy( leds, NUM_LEDS, 1);
    delay(8);
    FastLED.show();
  }
}



//This version of loop just reads a potentiometer to set a fractional position on the ring,
//swap for loop to test fractional positioning.
void loop2() {
  FastLED.clear();
  float analogIn = analogRead(A0);
  Serial.println(analogIn);
  float tempPos = analogIn / 1023.0 * 24.0;
  setPos(tempPos, 255);
  FastLED.show();
}
